/* gradient.c - renders gradients
 *
 *  Raster graphics library
 *
 *  Copyright (c) 1997 Alfredo K. Kojima
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *  
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *  
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "wraster.h"


RImage *renderHGradient(int width, int height, int r0, int g0, int b0, 
			int rf, int gf, int bf);
RImage *renderVGradient(int width, int height, int r0, int g0, int b0, 
			int rf, int gf, int bf);
RImage *renderDGradient(int width, int height, int r0, int g0, int b0, 
			int rf, int gf, int bf);


RImage*
RRenderGradient(int width, int height, RColor *from, RColor *to, int style)
{
    switch (style) {
     case WGRD_HORIZONTAL:
	return renderHGradient(width, height, from->red, from->green, 
			       from->blue, to->red, to->green, to->blue);
     case WGRD_VERTICAL:
	return renderVGradient(width, height, from->red, from->green, 
			       from->blue, to->red, to->green, to->blue);

     case WGRD_DIAGONAL:
	return renderDGradient(width, height, from->red, from->green, 
			       from->blue, to->red, to->green, to->blue);
    }
#ifdef DEBUG
    printf("invalid gradient style in RRenderGradient()");
#endif
    return NULL;
}


/*
 *----------------------------------------------------------------------
 * renderHGradient--
 * 	Renders a horizontal linear gradient of the specified size in the
 * RImage format with a border of the specified type. 
 * 
 * Returns:
 * 	A 24bit RImage with the gradient (no alpha channel).
 * 
 * Side effects:
 * 	None
 *---------------------------------------------------------------------- 
 */
RImage*
renderHGradient(int width, int height, int r0, int g0, int b0, 
		int rf, int gf, int bf)
{
    int i;
    float r, g, b, dr, dg, db;
    RImage *image;
    unsigned char *rp, *gp, *bp;

    image = RCreateImage(width, height, False);
    if (!image) {
	return NULL;
    }
    rp = image->data[0];
    gp = image->data[1];
    bp = image->data[2];

    r = (float)r0;
    g = (float)g0;
    b = (float)b0;
    
    dr = (float)(rf-r0)/(float)width;
    dg = (float)(gf-g0)/(float)width;
    db = (float)(bf-b0)/(float)width;
    /* render the first line */
    for (i=0; i<width; i++) {
	*(rp++) = (unsigned char)r;
	*(gp++) = (unsigned char)g;
	*(bp++) = (unsigned char)b;
	r += dr;
	g += dg;
	b += db;
    }
    /* copy the first line to the other lines */
    for (i=1; i<height; i++) {
	memcpy(&(image->data[0][i*width]), image->data[0], width);
	memcpy(&(image->data[1][i*width]), image->data[1], width);
	memcpy(&(image->data[2][i*width]), image->data[2], width);
    }
    return image;
}

/*
 *----------------------------------------------------------------------
 * renderVGradient--
 *      Renders a vertical linear gradient of the specified size in the
 * RImage format with a border of the specified type.
 *
 * Returns:
 *      A 24bit RImage with the gradient (no alpha channel).
 *
 * Side effects:
 *      None
 *----------------------------------------------------------------------
 */
RImage*
renderVGradient(int width, int height, int r0, int g0, int b0,
		int rf, int gf, int bf)
{
    int i;
    float r, g, b, dr, dg, db;
    RImage *image;
    unsigned char *rp, *gp, *bp;

    image = RCreateImage(width, height, False);
    if (!image) {
	return NULL;
    }    
    rp = image->data[0];
    gp = image->data[1];
    bp = image->data[2];
    
    r = (float)r0;
    g = (float)g0;
    b = (float)b0;

    dr = (float)(rf-r0)/(float)height;
    dg = (float)(gf-g0)/(float)height;
    db = (float)(bf-b0)/(float)height;

    for (i=0; i<height; i++) {
	memset(rp, (unsigned char)r, width);
	memset(gp, (unsigned char)g, width);
	memset(bp, (unsigned char)b, width);
	rp+=width;
	gp+=width;
	bp+=width;
        r+=dr;
        g+=dg;
        b+=db;
    }
    return image;
}


/*
 *----------------------------------------------------------------------
 * renderDGradient--
 *      Renders a diagonal linear gradient of the specified size in the
 * RImage format with a border of the specified type.
 *
 * Returns:
 *      A 24bit RImage with the gradient (no alpha channel).
 *
 * Side effects:
 *      None
 *----------------------------------------------------------------------
 */
RImage*
renderDGradient(int width, int height, int r0, int g0, int b0,
		int rf, int gf, int bf)
{
    int x, y;
    float from_red,from_green,from_blue;
    float to_red,to_green,to_blue;
    float t_r,t_g,t_b,d_r,d_g,d_b,r,g,b,dr,dg,db;
    RImage *image;
    unsigned char *rp, *gp, *bp;

    image = RCreateImage(width, height, False);
    if (!image) {
	return NULL;
    }
    rp = image->data[0];
    gp = image->data[1];
    bp = image->data[2];

    from_red = (float)r0;
    from_green = (float)g0;
    from_blue = (float)b0;

    to_red = (float)rf;
    to_green = (float)gf;
    to_blue = (float)bf;

    t_r = (from_red+to_red) / (float)2;
    t_g = (from_green+to_green) / (float)2;
    t_b = (from_blue+to_blue) / (float)2;

    d_r = (float)(height*2) / (float)(from_red+to_red);
    d_g = (float)(height*2) / (float)(from_green+to_green);
    d_b = (float)(height*2) / (float)(from_blue+to_blue);

    for (y=0; y<height; y++) {
        r = from_red;
        g = from_green;
        b = from_blue;

        dr = (t_r-r) / (float)width;
        dg = (t_g-g) / (float)width;
        db = (t_b-b) / (float)width;

        for (x=0; x<width; x++) {
            *(rp++) = (unsigned char)r;
            *(gp++) = (unsigned char)g;
            *(bp++) = (unsigned char)b;
            r+=dr;
            g+=dg;
            b+=db;
        }
        from_red+=d_r;
        from_green+=d_g;
        from_blue+=d_b;
        t_r+=d_r;
        t_g+=d_g;
        t_b+=d_b;
    }
    return image;
}

